#!/usr/bin/env python

#######################################################################
# Description : Sort integer (or float) with quicksort algorithm.
#
# Usage:
# quicksort.py [list of integer separate by space]
# quicksort.py 10 38 5900 -20 -15 ...
#
# quicksort.py -f [list of float separate by space]
# quicksort.py -f 25.9 14 48.5 ...
#
# By default it sort integer number.
#
# Author: Sebastien Denvil
# Contact: sebastien.denvil@ipsl.jussieu.fr
# $Date: 2004/02/13 14:57:18 $
# $Name: ipsl_cm4_v1_6 $
# $Revision: 1.2 $
#######################################################################

def partition(list, start, end):
    pivot = list[end]                          # Partition around the last value
    bottom = start-1                           # Start outside the area to be partitioned
    top = end                                  # Ditto

    done = 0
    while not done:                            # Until all elements are partitioned...

        while not done:                        # Until we find an out of place element...
            bottom = bottom+1                  # ... move the bottom up.

            if bottom == top:                  # If we hit the top...
                done = 1                       # ... we are done.
                break

            if list[bottom] > pivot:           # Is the bottom out of place?
                list[top] = list[bottom]       # Then put it at the top...
                break                          # ... and start searching from the top.

        while not done:                        # Until we find an out of place element...
            top = top-1                        # ... move the top down.
            
            if top == bottom:                  # If we hit the bottom...
                done = 1                       # ... we are done.
                break

            if list[top] < pivot:              # Is the top out of place?
                list[bottom] = list[top]       # Then put it at the bottom...
                break                          # ...and start searching from the bottom.

    list[top] = pivot                          # Put the pivot in its place.
    return top                                 # Return the split point


def quicksort(list, start, end):
    if start < end:                            # If there are two or more elements...
        split = partition(list, start, end)    # ... partition the sublist...
        quicksort(list, start, split-1)        # ... and sort both halves.
        quicksort(list, split+1, end)
    else:
        return

    
if __name__=="__main__":                       # If this script is run as a program:
    import sys, getopt
    # default value for TYPE
    TYPE = 'integer'
    # Extract the command-line arguments
    try:
        myopts, myargs = getopt.getopt(sys.argv[1:], 'f')
    except getopt.GetoptError:
        print 'Error in the command-line arguments'
        sys.exit(1)
    for myopt, myval in myopts:
        if myopt == '-f':
            TYPE = 'float'
    if TYPE == 'integer':
        try:
            list = map(int,myargs[0:])         # Get all the arguments as integer
        except ValueError:
            print 'You must give integer'
            sys.exit(1)
    else:
        try:
            list = map(float,myargs[0:])       # Get all the arguments as float
        except ValueError:
            print 'You must give float'
            sys.exit(1)
    start = 0
    end = len(list)-1
    quicksort(list,start,end)                  # Sort the entire list of arguments
    import string
    print string.join(map(str,list))           # Print out the sorted list
